#!perl
# 42-rerun-failed.t: Test::OnlySome::RerunFailed.
# Uses allkinds.yml, generated by running t/allkinds.test with
# prove -PTest::OnlySomeP and then changing the filename.

package t42;

use rlib 'lib';
use DTest;

use Test::OnlySome::RerunFailed filename=>'t/allkinds.yml';
    # not use_ok, so that the following will work.

os ok(0, 'Test 1 is skipped');
os ok(1, 'Test 2 is not skipped');
os ok(1, 'Test 3 is not skipped');
os ok(1, 'Test 4 is not skipped');
os ok(1, 'Test 5 is not skipped');
os ok(1, 'Test 6 is not skipped');

is_deeply $TEST_ONLYSOME, { n=>1, skip=>{1=>true, 3=>true, 4=>true}, verbose=>0 };

done_testing();
