#!perl
# 41-rerun-failed.t: Test::OnlySome::RerunFailed.
# Uses allkinds.yml, generated by running t/allkinds.test with
# prove -PTest::OnlySomeP and then changing the filename.

package t41;

use rlib 'lib';
use DTest;

BEGIN {
    use_ok 'Test::OnlySome::RerunFailed', filename=>'t/allkinds.yml';
}

# use_ok doesn't seem to play well with os for some reason, so the other tests
# are in t/42-rerun-failed.t.

done_testing();
